/*
 * Babu Liza Elena
 * 331CC
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "lcdpcf8574/lcdpcf8574.h"
#include "i2chw/i2cmaster.h"

#define NUM_NOTE		8

#define NO_BUTTON -1
#define BUTTON_0 0
#define BUTTON_1 1
#define BUTTON_2 2
#define BUTTON_3 3
#define BUTTON_4 4
#define BUTTON_5 5
#define BUTTON_6 6
#define BUTTON_7 7

#define DO1 262
#define RE 294
#define MI 330
#define FA 349
#define SOL 392
#define LA 440
#define SI 494
#define DO2 523

#define ERR -1

#define NO_STATE -1
#define PLAYING 0
#define LEARNING 1

#define SONG_LENGTH 58

volatile char current_state = NO_STATE;
volatile char button_pressed = NO_BUTTON;
volatile int current_octave = 6;
volatile long long int sum = 0;

int frecventa_nota[NUM_NOTE] = {
	262, 294, 330, 349, 392, 440, 494, 523
};

int joy_to_the_world[58] = {
	DO2, SI, LA, SOL, FA, MI, RE, DO1,
	SOL, LA, LA, SI, SI, DO2,
	DO2, DO2, SI, LA, SOL, SOL, FA, MI,
	DO2, DO2, SI, LA, SOL, SOL, FA, MI,
	MI, MI, MI, MI, FA, SOL, SOL, FA, MI, RE,
	RE, RE, RE, MI, FA, FA, MI, RE, DO1,
	DO2, LA, SOL, FA, MI, FA, MI, RE, DO1
};

int get_led(int i)
{
	switch (joy_to_the_world[i]) {
	case DO1:
		return PB0;
	case RE:
		return PB1;
	case MI:
		return PB2;
	case FA:
		return PB3;
	case SOL:
		return PB4;
	case LA:
		return PB5;
	case SI:
		return PB6;
	case DO2:
		return PB7;
	default:
		return ERR;
	}
}

int get_button(int i)
{
	switch (joy_to_the_world[i]) {
	case DO1:
		return BUTTON_0;
	case RE:
		return BUTTON_1;
	case MI:
		return BUTTON_2;
	case FA:
		return BUTTON_3;
	case SOL:
		return BUTTON_4;
	case LA:
		return BUTTON_5;
	case SI:
		return BUTTON_6;
	case DO2:
		return BUTTON_7;
	default:
		return ERR;
	}
}

void init_io()
{
	DDRA &= ~((1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | (1 << PA4) | (1 << PA5) | (1 << PA6) | (1 << PA7));
	PORTA |= ((1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | (1 << PA4) | (1 << PA5) | (1 << PA6) | (1 << PA7));
	
	DDRD &= ~((1 << PD0) | (1 << PD1)); 
	PORTD |= ((1 << PD0) | (1 << PD1));

	DDRB = 0xff;
	PORTB = 0;

	DDRD |= (1 << PD4);
	PORTD &= ~(1 << PD4);
	
	PCICR = (1 << PCIE3);
	PCICR |= (1 << PCIE0);
 
    PCMSK3 = (1 << PCINT24) | (1 << PCINT25);
    PCMSK0 = (1 << PCINT0) | (1 << PCINT1) | (1 << PCINT2) | (1 << PCINT3) |
    		 (1 << PCINT4) | (1 << PCINT5) | (1 << PCINT6) | (1 << PCINT7);
}

void speaker_reda(int nota)
{
	int perioada_nota = 1000000l / frecventa_nota[nota];
	int i;

	PORTD |= (1 << PD4);
	for (i = 0; i < perioada_nota / 2; ++i)
		_delay_us(1);

	PORTD &= ~(1 << PD4);
	for (i = 0; i < perioada_nota / 2; ++i)
		_delay_us(1);
}

ISR(PCINT0_vect)
{
	if ((PINA & (1 << PA0)) == 0) {
		button_pressed = BUTTON_0;
	} else if ((PINA & (1 << PA1)) == 0) {
		button_pressed = BUTTON_1;
	} else if ((PINA & (1 << PA2)) == 0) {
		button_pressed = BUTTON_2;
	} else if ((PINA & (1 << PA3)) == 0) {
		button_pressed = BUTTON_3;
	} else if ((PINA & (1 << PA4)) == 0) {
		button_pressed = BUTTON_4;
	} else if ((PINA & (1 << PA5)) == 0) {
		button_pressed = BUTTON_5;
	} else if ((PINA & (1 << PA6)) == 0) {
		button_pressed = BUTTON_6;
	} else if(((PINA & (1 << PA7)) == 0)) {
		button_pressed = BUTTON_7;
	}else {
		button_pressed = NO_BUTTON;
	}
}

ISR(PCINT3_vect)
{
	int i;

	if ((PIND & (1 << PD0)) == 0) {
		if (current_state == NO_STATE) {
			current_state = LEARNING;
		}
		else if (current_state == LEARNING) {
			current_state = PLAYING;
		}
		else {
			current_state = LEARNING;
		}
    } else if ((PIND & (1 << PD1)) == 0) {
    	current_octave = (current_octave + 1) % 8;
    	for (i = 0 ; i < NUM_NOTE; i++)
    		frecventa_nota[i] -= 100;
    	if (current_octave == 0) {
    		for (i = 0 ; i < NUM_NOTE; i++)
    		frecventa_nota[i] += 100 * 8;
    	}

    }
}

void do_playing()
{
	if (button_pressed == BUTTON_0) {
		PORTB |= (1 << PB0);
		speaker_reda(BUTTON_0);
	} else if (button_pressed == BUTTON_1) {
		PORTB |= (1 << PB1);
		speaker_reda(BUTTON_1);
	} else if (button_pressed == BUTTON_2) {
		PORTB |= (1 << PB2);
		speaker_reda(BUTTON_2);
	} else if (button_pressed == BUTTON_3) {
		PORTB |= (1 << PB3);
		speaker_reda(BUTTON_3);
	} else if (button_pressed == BUTTON_4) {
		PORTB |= (1 << PB4);
		speaker_reda(BUTTON_4);
	} else if (button_pressed == BUTTON_5) {
		PORTB |= (1 << PB5);
		speaker_reda(BUTTON_5);
	} else if (button_pressed == BUTTON_6) {
		PORTB |= (1 << PB6);
		speaker_reda(BUTTON_6);
	} else if (button_pressed == BUTTON_7) {
		PORTB |= (1 << PB7);
		speaker_reda(BUTTON_7);
	} else {
		PORTB = 0;
	}
}

void do_learning(void)
{
	int i, current_led, current_button;
	long int errors[SONG_LENGTH];
	char buffer[16];

	sum = 0;
	for (i = 0 ; i < SONG_LENGTH; i++) {
		errors[i] = 0;
		current_led = get_led(i);
		current_button = get_button(i);
		if (current_led == ERR || current_button == ERR)
			return;

		PORTB |= (1 << current_led);
		lcd_gotoxy(0, 0);
    	lcd_puts("Joy to the World!          ");
    	lcd_gotoxy(0, 1);
		switch (joy_to_the_world[i]) {
		case DO1:
			lcd_puts("DO ");
			break;
		case RE:
			lcd_puts("RE ");
			break;
		case MI:
			lcd_puts("MI ");
			break;
		case FA:
			lcd_puts("FA ");
			break;
		case SOL:
			lcd_puts("SOL");;
			break;
		case LA:
			lcd_puts("LA ");
			break;
		case SI:
			lcd_puts("SI ");
			break;
		case DO2:
			lcd_puts("DO ");
			break;
		}

		sprintf(buffer, "%d", current_octave);
		lcd_gotoxy(4, 1);
		lcd_puts("Octave ");
		lcd_gotoxy(11, 1);
		lcd_puts(buffer);
		lcd_gotoxy(12, 1);
		lcd_puts("    ");
		while(current_button != button_pressed) {
			errors[i]++;
			if (current_state == PLAYING) {
				lcd_gotoxy(0, 0);
			    lcd_puts("                          ");
			    lcd_gotoxy(0, 1);
			    lcd_puts("                          ");
				return;
			}
		}
		while(current_button == button_pressed) {
			speaker_reda(current_button);
		}
		PORTB = 0;
	}
	for (i = 0; i < SONG_LENGTH; i++) {
		sum += errors[i];
	}
}

int main(void)
{
	lcd_init(LCD_DISP_ON_BLINK);
	lcd_home();
	uint8_t led = 0;
    lcd_led(led);

	sei();
	init_io();

	while(1) {
		if (current_state == PLAYING) {
			do_playing();
		} else if (current_state == LEARNING) {
			lcd_gotoxy(0, 0);
    		lcd_puts("Joy to the World");
			sum /= (1000 * SONG_LENGTH);
			if (sum != 0 && sum < 10 * SONG_LENGTH) {
				lcd_gotoxy(0, 1);
			    lcd_puts("   Very good!   ");
			} else if (10 * SONG_LENGTH <= sum && sum < 20 * SONG_LENGTH) {
				lcd_gotoxy(0, 1);
			    lcd_puts("   Good!        ");
			} else if (sum != 0) {
				lcd_gotoxy(0, 1);
			    lcd_puts("   Ok!          ");
			}
			_delay_ms(1000);
			do_learning();
		} else {
			lcd_gotoxy(0, 0);
		    lcd_puts("Musica Maestro");
		    lcd_gotoxy(0, 1);
		    lcd_puts("Liza Babu, 331CC");
		}
	}
	
	return 0;
}
